@@ -22,15 +22,21 @@ from __future__ import annotations
 
 import codecs
 import logging
-import pickle
 from io import BytesIO
 from itertools import starmap
+from typing import TYPE_CHECKING, ClassVar, Literal
 
+from translate.lang.data import plural_tags
 from translate.misc.multistring import multistring
 from translate.storage.placeables import StringElem
 from translate.storage.placeables import parse as rich_parse
 from translate.storage.workflow import StateEnum as states
 
+if TYPE_CHECKING:
+    from collections.abc import Callable
+
+logger = logging.getLogger(__name__)
+
 # Simple BOM based encoding detection
 ENCODING_BOMS = (
     (codecs.BOM_UTF8, "utf-8-sig"),
@@ -42,6 +48,8 @@ ENCODING_BOMS = (
     (codecs.BOM_UTF32_LE, "utf-32-le"),
 )
 
+MISSING = object()
+
 
 class ParseError(Exception):
     def __init__(self, inner_exc):
@@ -76,7 +84,7 @@ class TranslationUnit:
 
     """
 
-    rich_parsers = []
+    rich_parsers: list[Callable[[str], list[StringElem]]] = []
     """A list of functions to use for parsing a string into a rich string
     tree."""
 
@@ -111,7 +119,7 @@ class TranslationUnit:
     # """
     #
     # ... but by default a format will not support state:
-    STATE = {}
+    STATE: dict[states, tuple[states, states]] = {}
 
     _store = None
     _source = None
@@ -146,10 +154,10 @@ class TranslationUnit:
         return not self.__eq__(other)
 
     def __hash__(self):
-        return None
+        return hash((self.source, self.target, self.getid()))
 
     def __str__(self):
-        """Converts to a string representation. Most often overriden by subclasses."""
+        """Converts to a string representation. Most often overridden by subclasses."""
         # no point in showing store object.
         return ", ".join(
             f"{k}: {self.__dict__[k]}"
@@ -277,7 +285,7 @@ class TranslationUnit:
         """
         Sets the unique identified for this unit.
 
-        only implemented if format allows ids independant from other unit
+        only implemented if format allows ids independent from other unit
         properties like source or context
         """
 
@@ -322,8 +330,7 @@ class TranslationUnit:
         else:
             self.addlocation(location)
 
-    @staticmethod
-    def getcontext():
+    def getcontext(self):
         """Get the message context."""
         return ""
 
@@ -354,7 +361,7 @@ class TranslationUnit:
                        - 'developer', 'programmer', 'source code' (synonyms)
         """
         if position == "append" and getattr(self, "notes", None):
-            self.notes += "\n" + text
+            self.notes += f"\n{text}"
         else:
             self.notes = text
 
@@ -408,6 +415,9 @@ class TranslationUnit:
         """
         return bool(self.source)
 
+    def marktranslatable(self, value: bool) -> None:
+        """Marks the unit as translatable or not."""
+
     @staticmethod
     def isfuzzy():
         """Indicates whether this unit is fuzzy."""
@@ -425,16 +435,16 @@ class TranslationUnit:
         """Make a unit obsolete."""
 
     @staticmethod
-    def isheader():
+    def isheader() -> bool:
         """Indicates whether this unit is a header."""
         return False
 
     @staticmethod
-    def isreview():
+    def isreview() -> bool:
         """Indicates whether this unit needs review."""
         return False
 
-    def isblank(self):
+    def isblank(self) -> bool:
         """
         Used to see if this unit has no source or target string.
 
@@ -545,14 +555,14 @@ class TranslationUnit:
 class TranslationStore:
     """Base class for stores for multiple translation units of type UnitClass."""
 
-    UnitClass = TranslationUnit
+    UnitClass: ClassVar[type[TranslationUnit]] = TranslationUnit
     """The class of units that will be instantiated and used by this class"""
     Name = "Base translation store"
     """The human usable name of this store type"""
-    Mimetypes = None
+    Mimetypes: ClassVar[list[str]] = []
     """A list of MIME types associated with this store type"""
-    Extensions = None
-    """A list of file extentions associated with this store type"""
+    Extensions: ClassVar[list[str]] = []
+    """A list of file extensions associated with this store type"""
     _binary = False
     """Indicates whether a file should be accessed as a binary file."""
     suggestions_in_format = False
@@ -710,15 +720,11 @@ class TranslationStore:
             remove_source(unit.source)
 
         for location in unit.getlocations():
-            if (
-                location in self.locationindex
-                and self.locationindex[location] is not None
-                and self.locationindex[location] == unit
-            ):
+            if location in self.locationindex and self.locationindex[location] == unit:
                 del self.locationindex[location]
 
     def add_unit_to_index(self, unit):
-        """Add a unit to source and location idexes."""
+        """Add a unit to source and location indices."""
         self.id_index[unit.getid()] = unit
 
         def insert_unit(source):
@@ -779,8 +785,16 @@ class TranslationStore:
         Converts to a bytes representation that can be parsed back using
         :meth:`~.TranslationStore.parsestring`.
         `out` should be an open file-like objects to write to.
+
+        .. note::
+
+           This method should be overridden by subclasses to provide
+           format-specific serialization.
         """
-        out.write(pickle.dumps(self))
+        raise NotImplementedError(
+            f"{self.__class__.__name__} does not implement serialize(). "
+            "Subclasses must provide their own serialization method."
+        )
 
     def isempty(self):
         """Return True if the object doesn't contain any translation units."""
@@ -821,7 +835,7 @@ class TranslationStore:
 
     def detect_encoding(
         self, text: bytes, default_encodings: list[str] | None = None
-    ) -> tuple[str, str]:
+    ) -> tuple[str | None, str | None]:
         """
         Try to detect a file encoding from `text`, using either the chardet lib
         or by trying to decode the file.
@@ -829,7 +843,8 @@ class TranslationStore:
         if not default_encodings:
             default_encodings = ["utf-8"]
         try:
-            from charset_normalizer import detect
+            # pylint: disable-next=import-outside-toplevel
+            from charset_normalizer import detect  # noqa: PLC0415
         except ImportError:
             detected_encoding = self.fallback_detection(text)
         else:
@@ -863,7 +878,7 @@ class TranslationStore:
             if encoding == self.encoding and suffix == "sig":
                 encodings.append(detected_encoding["encoding"])
             elif detected_encoding["encoding"] != self.encoding:
-                logging.warning(
+                logger.warning(
                     "trying to parse %s with encoding: %s but "
                     "detected encoding is %s (confidence: %s)",
                     self.filename,
@@ -888,8 +903,18 @@ class TranslationStore:
         return r_text, r_encoding
 
     def parse(self, data):
-        """Parser to process the given source string."""
-        self.units = pickle.loads(data).units
+        """
+        Parser to process the given source string.
+
+        .. note::
+
+           This method should be overridden by subclasses to provide
+           format-specific parsing.
+        """
+        raise NotImplementedError(
+            f"{self.__class__.__name__} does not implement parse(). "
+            "Subclasses must provide their own parsing method."
+        )
 
     def savefile(self, storefile):
         """Write the string representation to the given file (or filename)."""
@@ -914,6 +939,14 @@ class TranslationStore:
             fileobj = open(filename, "wb")
         self.savefile(fileobj)
 
+    @classmethod
+    def _from_handle(cls, storehandle):
+        storestring = storehandle.read()
+        newstore = cls.parsestring(storestring)
+        newstore.fileobj = storehandle
+        newstore._assignname()
+        return newstore
+
     @classmethod
     def parsefile(cls, storefile):
         """
@@ -921,16 +954,16 @@ class TranslationStore:
         to an object.
         """
         if isinstance(storefile, str):
-            storefile = open(storefile, "rb")
+            with open(storefile, "rb") as storehandle:
+                return cls._from_handle(storehandle)
         mode = getattr(storefile, "mode", "rb")
         # For some reason GzipFile returns 1, so we have to test for that here
         if mode == 1 or "r" in mode:
-            storestring = storefile.read()
+            newstore = cls._from_handle(storefile)
             storefile.close()
-            newstore = cls.parsestring(storestring)
-        else:
-            storestring = ""
-            newstore = cls()
+            return newstore
+
+        newstore = cls()
         newstore.fileobj = storefile
         newstore._assignname()
         return newstore
@@ -945,6 +978,19 @@ class TranslationStore:
         """
         return "id"
 
+    def get_base_locale_code(self) -> str:
+        locale = self.gettargetlanguage()
+        if not locale:
+            return "en"
+        # Handle b+ style language codes and standardize
+        return (
+            locale.removeprefix("b+").replace("_", "-").replace("+", "-").split("-")[0]
+        )
+
+    def get_plural_tags(self) -> list[str]:
+        locale = self.get_base_locale_code()
+        return plural_tags.get(locale, plural_tags["en"])
+
 
 class UnitId:
     KEY_SEPARATOR = "."
@@ -967,11 +1013,14 @@ class UnitId:
         return self.__class__([*self.parts, (key, value)])
 
     @classmethod
-    def from_string(cls, text):
-        result = []
+    def from_key(cls, text: str) -> UnitId:
+        return cls([("key", text)])
+
+    @classmethod
+    def from_string(cls, text: str) -> UnitId:
+        result: list[tuple[Literal["key", "index"], str | int]] = []
         # Strip possible leading separator
-        if text.startswith(cls.KEY_SEPARATOR):
-            text = text[len(cls.KEY_SEPARATOR) :]
+        text = text.removeprefix(cls.KEY_SEPARATOR)
         for item in text.split(cls.KEY_SEPARATOR):
             bracepos = item.find("[")
             endbracepos = item.find("]")
@@ -1013,6 +1062,8 @@ class DictUnit(TranslationUnit):
         return self._unitid
 
     def storevalue(self, output, value, override_key=None, unset=False):
+        if isinstance(value, multistring):
+            value = str(value)
         parent = target = output
         parts = self.get_unitid().parts
         key = None
@@ -1031,6 +1082,9 @@ class DictUnit(TranslationUnit):
             if not use_list and isinstance(target[key], list):
                 # Convert list to dict if needed
                 target[key] = dict(enumerate(target[key]))
+            elif use_list and isinstance(target[key], dict):
+                # Replace with an empty list if needed (this loses previous content)
+                target[key] = []
             # Handle placeholders
             if target[key] is None:
                 target[key] = default.copy()
@@ -1046,7 +1100,7 @@ class DictUnit(TranslationUnit):
                 # Remove empty dict from parent
                 if not target and key:
                     del parent[key]
-            else:
+            elif target.get(child_key, MISSING) != value:
                 target[child_key] = value
         elif child_element == "index":
             if len(target) <= child_key:
