@@ -25,6 +25,8 @@ parsed rich-string tree. It is the base class of all placeables.
 import contextlib
 import logging
 
+logger = logging.getLogger(__name__)
+
 
 def filter_all(e):
     return True
@@ -52,7 +54,7 @@ class StringElem:
     has_content = True
     """Whether this string can have sub-elements."""
     iseditable = True
-    """Whether this string should be changable by the user. Not used at
+    """Whether this string should be changeable by the user. Not used at
     the moment."""
     isfragile = False
     """Whether this element should be deleted in its entirety when partially
@@ -82,7 +84,7 @@ class StringElem:
 
         for key, value in kwargs.items():
             if hasattr(self, key):
-                raise ValueError("attribute already exists: %s" % (key))
+                raise ValueError(f"attribute already exists: {key}")
             setattr(self, key, value)
 
     # SPECIAL METHODS #
@@ -121,10 +123,6 @@ class StringElem:
         """Emulate the ``unicode`` class."""
         return str(self)[i]
 
-    def __getslice__(self, i, j):
-        """Emulate the ``unicode`` class."""
-        return str(self)[i:j]
-
     def __gt__(self, rhs):
         """Emulate the ``unicode`` class."""
         return str(self) > rhs
@@ -162,13 +160,13 @@ class StringElem:
 
     def __repr__(self):
         elemstr = ", ".join(repr(elem) for elem in self.sub)
-        return "<%(class)s(%(id)s%(rid)s%(xid)s[%(subs)s])>" % {
-            "class": self.__class__.__name__,
-            "id": self.id is not None and 'id="%s" ' % (self.id) or "",
-            "rid": self.rid is not None and 'rid="%s" ' % (self.rid) or "",
-            "xid": self.xid is not None and 'xid="%s" ' % (self.xid) or "",
-            "subs": elemstr,
-        }
+        return "<{class_name}({id}{rid}{xid}[{subs}])>".format(
+            class_name=self.__class__.__name__,
+            id=(self.id is not None and f'id="{self.id}" ') or "",
+            rid=(self.rid is not None and f'rid="{self.rid}" ') or "",
+            xid=(self.xid is not None and f'xid="{self.xid}" ') or "",
+            subs=elemstr,
+        )
 
     def __str__(self):
         if callable(self.renderer):
@@ -186,9 +184,9 @@ class StringElem:
                   string or unicode.
         """
         for elem in self.flatten():
-            for i in range(len(elem.sub)):
-                if isinstance(elem.sub[i], str):
-                    elem.sub[i] = f(elem.sub[i])
+            for i, subelement in enumerate(elem.sub):
+                if isinstance(subelement, str):
+                    elem.sub[i] = f(subelement)
 
     def copy(self):
         """
@@ -197,7 +195,7 @@ class StringElem:
 
         .. note:: ``self.renderer`` is **not** copied.
         """
-        # logging.debug('Copying instance of class %s' % (self.__class__.__name__))
+        # logger.debug('Copying instance of class %s' % (self.__class__.__name__))
         cp = self.__class__(id=self.id, xid=self.xid, rid=self.rid)
         for sub in self.sub:
             if isinstance(sub, StringElem):
@@ -214,8 +212,8 @@ class StringElem:
         if parent is None:
             raise ElementNotFoundError(repr(elem))
         subidx = -1
-        for i in range(len(parent.sub)):
-            if parent.sub[i] is elem:
+        for i, subelement in enumerate(parent.sub):
+            if subelement is elem:
                 subidx = i
                 break
         if subidx < 0:
@@ -239,13 +237,11 @@ class StringElem:
         if start_index == end_index:
             return StringElem(), self, 0
         if start_index > end_index:
-            raise IndexError(
-                "start_index > end_index: %d > %d" % (start_index, end_index)
-            )
+            raise IndexError(f"start_index > end_index: {start_index} > {end_index}")
         if start_index < 0 or start_index > len(self):
-            raise IndexError("start_index: %d" % (start_index))
+            raise IndexError(f"start_index: {start_index}")
         if end_index < 1 or end_index > len(self) + 1:
-            raise IndexError("end_index: %d" % (end_index))
+            raise IndexError(f"end_index: {end_index}")
 
         start = self.get_index_data(start_index)
         if isinstance(start["elem"], tuple):
@@ -260,7 +256,7 @@ class StringElem:
         assert start["elem"].isleaf()
         assert end["elem"].isleaf()
 
-        # logging.debug('FROM %s TO %s' % (start, end))
+        # logger.debug('FROM %s TO %s' % (start, end))
 
         # Ranges can be one of 3 types:
         # 1) The entire string.
@@ -271,7 +267,7 @@ class StringElem:
 
         # Case 1: Entire string #
         if start_index == 0 and end_index == len(self):
-            # logging.debug('Case 1: [%s]' % (unicode(self)))
+            # logger.debug('Case 1: [%s]' % (unicode(self)))
             removed = self.copy()
             self.sub = []
             return removed, None, None
@@ -281,8 +277,7 @@ class StringElem:
             start["elem"] is end["elem"]
             and start["offset"] == 0
             and end["offset"] == len(start["elem"])
-            or (not start["elem"].iseditable and start["elem"].isfragile)
-        ):
+        ) or (not start["elem"].iseditable and start["elem"].isfragile):
             ##### FOR DEBUGGING #####
             # s = ''
             # for e in self.flatten():
@@ -290,7 +285,7 @@ class StringElem:
             #        s += '[' + unicode(e) + ']'
             #    else:
             #        s += unicode(e)
-            # logging.debug('Case 2: %s' % (s))
+            # logger.debug('Case 2: %s' % (s))
             #########################
 
             if start["elem"] is self and self.__class__ is StringElem:
@@ -301,7 +296,7 @@ class StringElem:
             parent = self.get_parent_elem(start["elem"])
             offset = parent.elem_offset(start["elem"])
             # Filter out start['elem'] below with a list comprehension in stead
-            # of using parent.sub.remove(), becase list.remove() tests value
+            # of using parent.sub.remove(), because list.remove() tests value
             # and not identity, which is what we want here. This ensures that
             # start['elem'] is removed and not the first element that is equal
             # to it.
@@ -321,7 +316,7 @@ class StringElem:
             #        )
             #    else:
             #        s += unicode(e)
-            # logging.debug('Case 3: %s' % (s))
+            # logger.debug('Case 3: %s' % (s))
             #########################
 
             # XXX: This might not have the expected result if start['elem']
@@ -340,16 +335,16 @@ class StringElem:
         range_nodes = self.depth_first()
         startidx = 0
         endidx = -1
-        for i in range(len(range_nodes)):
-            if range_nodes[i] is start["elem"]:
+        for i, range_node in enumerate(range_nodes):
+            if range_node is start["elem"]:
                 startidx = i
-            elif range_nodes[i] is end["elem"]:
+            elif range_node is end["elem"]:
                 endidx = i
                 break
         range_nodes = range_nodes[startidx : endidx + 1]
         # assert (range_nodes[0] is start['elem'] and
         #        range_nodes[-1] is end['elem'])
-        # logging.debug("Nodes in delete range: %s" % (str(range_nodes)))
+        # logger.debug("Nodes in delete range: %s" % (str(range_nodes)))
 
         marked_nodes = []  # Contains nodes that have been marked for deletion (directly or inderectly (via parent)).
         for node in range_nodes[1:-1]:
@@ -357,7 +352,7 @@ class StringElem:
                 continue
             subtree = node.depth_first()
             if not [e for e in subtree if e is end["elem"]]:
-                # logging.debug("Marking node: %s" % (subtree))
+                # logger.debug("Marking node: %s" % (subtree))
                 marked_nodes.extend(subtree)  # "subtree" does not include "node"
 
         ##### FOR DEBUGGING #####
@@ -369,7 +364,7 @@ class StringElem:
         #        s += '%s]%s' % (e[:end['offset']], e[end['offset']:])
         #    else:
         #        s += unicode(e)
-        # logging.debug('Case 4: %s' % (s))
+        # logger.debug('Case 4: %s' % (s))
         #########################
 
         removed = self.copy()
@@ -543,7 +538,7 @@ class StringElem:
         string (Unicode) representation.
         """
         if offset < 0 or offset > len(self):
-            raise IndexError("Index out of range: %d" % (offset))
+            raise IndexError(f"Index out of range: {offset}")
         if isinstance(text, str):
             text = StringElem(text)
         if not isinstance(text, StringElem):
@@ -573,12 +568,12 @@ class StringElem:
         if offset == 0:
             # 1.1 #
             if oelem.iseditable:
-                # logging.debug('Case 1.1')
+                # logger.debug('Case 1.1')
                 oelem.sub.insert(0, checkleaf(oelem, text))
                 oelem.prune()
                 return True
             # 1.2 #
-            # logging.debug('Case 1.2')
+            # logger.debug('Case 1.2')
             oparent = self.get_ancestor_where(oelem, filter_editable)
             if oparent is not None:
                 oparent.sub.insert(0, checkleaf(oparent, text))
@@ -588,7 +583,7 @@ class StringElem:
 
         # Case 2 #
         if offset == len(self):
-            # logging.debug('Case 2')
+            # logger.debug('Case 2')
             last = self.flatten()[-1]
             parent = self.get_ancestor_where(last, filter_editable)
             if parent is None:
@@ -611,7 +606,7 @@ class StringElem:
         # Case 3 #
         if oelem is before:
             if oelem.iseditable:
-                # logging.debug('Case 3')
+                # logger.debug('Case 3')
                 eoffset = offset - self.elem_offset(oelem)
                 if oelem.isleaf():
                     s = str(oelem)  # Collapse all sibling strings into one
@@ -628,7 +623,7 @@ class StringElem:
         # And the only case left: Case 4 #
         # 4.1 #
         if not before.iseditable and not oelem.iseditable:
-            # logging.debug('Case 4.1')
+            # logger.debug('Case 4.1')
             # Neither are editable, so we add it as a sibling (to the right)
             # of before
             bparent = self.get_parent_elem(before)
@@ -648,7 +643,7 @@ class StringElem:
 
         # 4.2 #
         if before.iseditable and oelem.iseditable:
-            # logging.debug('Case 4.2')
+            # logger.debug('Case 4.2')
             # We can add to either, but we try hard to add to the correct one
             # so that we avoid inserting text in the wrong place on undo, for
             # example.
@@ -661,7 +656,7 @@ class StringElem:
             if oelem_type == preferred_type and before_type != preferred_type:
                 # oelem has the right type and before has the wrong type
                 return oelem.insert(0, text)
-            if preferred_type not in (oelem_type, before_type):
+            if preferred_type not in {oelem_type, before_type}:
                 # Both are the wrong type, so we add it as if neither were
                 # editable
                 bparent = self.get_parent_elem(before)
@@ -678,12 +673,12 @@ class StringElem:
 
         # 4.3 #
         if before.iseditable and not oelem.iseditable:
-            # logging.debug('Case 4.3')
+            # logger.debug('Case 4.3')
             return before.insert(len(before), text)  # Reinterpret as a case 2
 
         # 4.4 #
         if not before.iseditable and oelem.iseditable:
-            # logging.debug('Case 4.4')
+            # logger.debug('Case 4.4')
             return oelem.insert(0, text)  # Reinterpret as a case 1
 
         return False
@@ -709,7 +704,7 @@ class StringElem:
             text = StringElem(text)
 
         if left is right:
-            # logging.debug('left%s.sub.append(%s)' % (repr(left), repr(text)))
+            # logger.debug('left%s.sub.append(%s)' % (repr(left), repr(text)))
             left.sub.append(text)
             return True
         # XXX: The "in" keyword is *not* used below, because the "in" tests
@@ -718,13 +713,13 @@ class StringElem:
 
         if left is None:
             if self is right:
-                # logging.debug('self%s.sub.insert(0, %s)' %
+                # logger.debug('self%s.sub.insert(0, %s)' %
                 #              (repr(self), repr(text)))
                 self.sub.insert(0, text)
                 return True
             parent = self.get_parent_elem(right)
             if parent is not None:
-                # logging.debug('parent%s.sub.insert(0, %s)' %
+                # logger.debug('parent%s.sub.insert(0, %s)' %
                 #              (repr(parent), repr(text)))
                 parent.sub.insert(0, text)
                 return True
@@ -732,13 +727,13 @@ class StringElem:
 
         if right is None:
             if self is left:
-                # logging.debug('self%s.sub.append(%s)' %
+                # logger.debug('self%s.sub.append(%s)' %
                 #              (repr(self), repr(text)))
                 self.sub.append(text)
                 return True
             parent = self.get_parent_elem(left)
             if parent is not None:
-                # logging.debug('parent%s.sub.append(%s)' %
+                # logger.debug('parent%s.sub.append(%s)' %
                 #              (repr(parent), repr(text)))
                 parent.sub.append(text)
                 return True
@@ -753,7 +748,7 @@ class StringElem:
                 ischild = True
                 break
         if ischild:
-            # logging.debug('left%s.sub.insert(0, %s)' %
+            # logger.debug('left%s.sub.insert(0, %s)' %
             #              (repr(left), repr(text)))
             left.sub.insert(0, text)
             return True
@@ -764,7 +759,7 @@ class StringElem:
                 ischild = True
                 break
         if ischild:
-            # logging.debug('right%s.sub.append(%s)' %
+            # logger.debug('right%s.sub.append(%s)' %
             #              (repr(right), repr(text)))
             right.sub.append(text)
             return True
@@ -776,7 +771,7 @@ class StringElem:
                 if child is left:
                     break
                 idx += 1
-            # logging.debug('parent%s.sub.insert(%d, %s)' %
+            # logger.debug('parent%s.sub.insert(%d, %s)' %
             #              (repr(parent), idx, repr(text)))
             parent.sub.insert(idx, text)
             return True
@@ -788,12 +783,12 @@ class StringElem:
                 if child is right:
                     break
                 idx += 1
-            # logging.debug('parent%s.sub.insert(%d, %s)' %
+            # logger.debug('parent%s.sub.insert(%d, %s)' %
             #              (repr(parent), idx, repr(text)))
             parent.sub.insert(0, text)
             return True
 
-        logging.debug("Could not insert between %r and %r... odd.", left, right)
+        logger.debug("Could not insert between %r and %r... odd.", left, right)
         return False
 
     def isleaf(self):
@@ -857,7 +852,7 @@ class StringElem:
         indent_prefix = " " * indent * 2
         out = f"{indent_prefix}{self.__class__.__name__} [{self!s}]"
         if verbose:
-            out += " " + repr(self)
+            out += f" {self!r}"
 
         print(out)
 
