@@ -53,7 +53,7 @@ Java
     MessageFormat, but this may be implemented in future.
 
     All delimiter types, comments, line continuations and spaces handling in
-    delimeters are supported.
+    delimiters are supported.
 
 Mozilla
     Mozilla files use '=' as a delimiter, are UTF-8 encoded and thus don't
@@ -136,8 +136,16 @@ from copy import deepcopy
 from lxml import etree
 
 from translate.lang import data
-from translate.misc import quote
 from translate.misc.multistring import multistring
+from translate.misc.quote import (
+    java_utf8_properties_encode,
+    javapropertiesencode,
+    mozillaescapemarginspaces,
+    propertiesdecode,
+    rstripeol,
+    xwiki_properties_decode,
+    xwiki_properties_encode,
+)
 from translate.storage import base
 
 labelsuffixes = (".label", ".title")
@@ -154,7 +162,7 @@ ending in :attr:`.labelsuffixes` into accelerator notation"""
 eol = "\n"
 
 
-def is_line_continuation(line):
+def is_line_continuation(line: str) -> bool:
     r"""
     Determine whether *line* has a line continuation marker.
 
@@ -199,7 +207,7 @@ def get_comment_one_line(line):
     return None
 
 
-def get_comment_start(line):
+def get_comment_start(line: str) -> str | None:
     """
     Determine whether a *line* starts a new multi-line comment.
 
@@ -214,7 +222,7 @@ def get_comment_start(line):
     return None
 
 
-def get_comment_end(line):
+def get_comment_end(line: str) -> str | None:
     """
     Determine whether a *line* ends a new multi-line comment.
 
@@ -229,7 +237,7 @@ def get_comment_end(line):
     return None
 
 
-def _key_strip(key):
+def _key_strip(key: str) -> str:
     """
     Cleanup whitespace found around a key.
 
@@ -245,11 +253,11 @@ def _key_strip(key):
     return newkey.lstrip()
 
 
-dialects = {}
+dialects: dict[str, type[Dialect]] = {}
 default_dialect = "java"
 
 
-def register_dialect(dialect):
+def register_dialect(dialect: type[Dialect]) -> type[Dialect]:
     """Decorator that registers the dialect."""
     dialects[dialect.name] = dialect
     return dialect
@@ -262,30 +270,30 @@ def get_dialect(dialect=default_dialect):
 class Dialect:
     """Settings for the various behaviours in key=value files."""
 
-    name = None
-    default_encoding = "iso-8859-1"
+    name: str
+    default_encoding: str = "iso-8859-1"
     delimiters: list[str] = []
-    pair_terminator = ""
-    key_wrap_char = ""
-    value_wrap_char = ""
-    drop_comments = []
-    has_plurals = False
+    pair_terminator: str = ""
+    key_wrap_char: str = ""
+    value_wrap_char: str = ""
+    drop_comments: list[str] = []
+    has_plurals: bool = False
 
     @staticmethod
-    def encode(string, encoding=None):
+    def encode(string: str, encoding: str | None = None) -> str:
         """Encode the string."""
         # FIXME: dialects are a bad idea, not possible for subclasses
         # to override key methods
         if encoding not in {"utf-8", "utf-16"}:
-            return quote.javapropertiesencode(string or "")
-        return quote.java_utf8_properties_encode(string or "")
+            return javapropertiesencode(string or "", encoding)
+        return java_utf8_properties_encode(string or "")
 
     @staticmethod
-    def decode(string):
-        return quote.propertiesdecode(string)
+    def decode(string: str) -> str:
+        return propertiesdecode(string)
 
     @classmethod
-    def find_delimiter(cls, line):
+    def find_delimiter(cls, line: str) -> tuple[str | None, int]:
         """
         Find the type and position of the delimiter in a property line.
 
@@ -306,16 +314,20 @@ class Dialect:
         delimiters = delimiter_dict
         # Figure out starting position
         start_pos = len(line) - len(line.lstrip())  # Skip initial whitespace
-        if cls.key_wrap_char and line[start_pos] == cls.key_wrap_char:
+        if (
+            cls.key_wrap_char
+            and start_pos < len(line)
+            and line[start_pos] == cls.key_wrap_char
+        ):
             # Skip the key if it is delimited by some char
             start_pos += 1
             while line[start_pos] != cls.key_wrap_char or line[start_pos - 1] == "\\":
                 start_pos += 1
         # Find the position of each delimiter type
-        for delimiter in delimiters:
+        for delimiter, value in delimiters.items():
             pos = line.find(delimiter, start_pos)
             while pos != -1:
-                if delimiters[delimiter] == -1 and line[pos - 1] != "\\":
+                if value == -1 and line[pos - 1] != "\\":
                     delimiters[delimiter] = pos
                     break
                 pos = line.find(delimiter, pos + 1)
@@ -331,42 +343,42 @@ class Dialect:
         if mindelimiter is None and delimiters.get(" ", -1) != -1:
             # Use space delimiter if we found nothing else
             return (" ", delimiters[" "])
+        # If space delimiter occurs earlier than ":" or "=" then it is the
+        # delimiter only if there are non-whitespace characters between it and
+        # the other detected delimiter.
         if (
             mindelimiter is not None
             and " " in delimiters
             and delimiters[" "] < delimiters[mindelimiter]
+            and len(line[delimiters[" "] : delimiters[mindelimiter]].strip()) > 0
         ):
-            # If space delimiter occurs earlier than ":" or "=" then it is the
-            # delimiter only if there are non-whitespace characters between it and
-            # the other detected delimiter.
-            if len(line[delimiters[" "] : delimiters[mindelimiter]].strip()) > 0:
-                return (" ", delimiters[" "])
+            return (" ", delimiters[" "])
         return (mindelimiter, minpos)
 
     @staticmethod
-    def key_strip(key):
+    def key_strip(key: str) -> str:
         """Strip unneeded characters from the key."""
         return _key_strip(key)
 
     @staticmethod
-    def value_strip(value):
+    def value_strip(value: str) -> str:
         """Strip unneeded characters from the value."""
         return value.lstrip()
 
     @staticmethod
-    def is_line_continuation(line):
+    def is_line_continuation(line: str) -> bool:
         return is_line_continuation(line)
 
     @staticmethod
-    def strip_line_continuation(value):
+    def strip_line_continuation(value: str) -> str:
         return value[:-1]
 
     @staticmethod
-    def get_key_cldr_name(key):
+    def get_key_cldr_name(key: str) -> tuple[str, str]:
         return (key, "other")
 
     @staticmethod
-    def get_cldr_names_order():
+    def get_cldr_names_order() -> list[str]:
         return ["other"]
 
     @staticmethod
@@ -389,7 +401,7 @@ class DialectJavaUtf8(DialectJava):
 
     @staticmethod
     def encode(string, encoding=None):
-        return quote.java_utf8_properties_encode(string or "")
+        return java_utf8_properties_encode(string or "")
 
 
 @register_dialect
@@ -400,7 +412,7 @@ class DialectJavaUtf16(DialectJava):
 
     @staticmethod
     def encode(string, encoding=None):
-        return quote.java_utf8_properties_encode(string or "")
+        return java_utf8_properties_encode(string or "")
 
 
 @register_dialect
@@ -418,11 +430,11 @@ class DialectXWiki(DialectJava):
 
     @staticmethod
     def encode(string, encoding=None):
-        return quote.xwiki_properties_encode(string or "", encoding)
+        return xwiki_properties_encode(string or "", encoding)
 
     @staticmethod
     def decode(string):
-        return quote.xwiki_properties_decode(string)
+        return xwiki_properties_decode(string)
 
 
 @register_dialect
@@ -439,8 +451,8 @@ class DialectMozilla(DialectJavaUtf8):
     @staticmethod
     def encode(string, encoding=None):
         """Encode the string."""
-        string = quote.java_utf8_properties_encode(string or "")
-        return quote.mozillaescapemarginspaces(string or "")
+        string = java_utf8_properties_encode(string or "")
+        return mozillaescapemarginspaces(string or "")
 
 
 @register_dialect
@@ -498,9 +510,9 @@ class DialectGwt(DialectJavaUtf8):
     @classmethod
     def encode(cls, string, encoding=None):
         if encoding not in {"utf-8", "utf-16"}:
-            result = quote.javapropertiesencode(string or "")
+            result = javapropertiesencode(string or "")
         else:
-            result = quote.java_utf8_properties_encode(string or "")
+            result = java_utf8_properties_encode(string or "")
         return result.replace("'", "''")
 
     @classmethod
@@ -523,7 +535,7 @@ class DialectSkype(Dialect):
 
     @staticmethod
     def encode(string, encoding=None):
-        return quote.java_utf8_properties_encode(string or "")
+        return java_utf8_properties_encode(string or "")
 
 
 @register_dialect
@@ -571,8 +583,8 @@ class DialectStrings(Dialect):
 
     @staticmethod
     def is_line_continuation(line):
-        l = line.rstrip()
-        return not l or l[-1] != ";"
+        stripped = line.rstrip()
+        return not stripped or stripped[-1] != ";"
 
     @staticmethod
     def strip_line_continuation(value):
@@ -606,12 +618,6 @@ class proppluralunit(base.TranslationUnit):
         existing = self.units.keys()
         return [key for key in data.cldr_plural_categories if key in existing]
 
-    def _get_target_mapping(self):
-        cldr_mapping = proppluralunit._get_language_mapping(self._store.targetlanguage)
-        if cldr_mapping:
-            return cldr_mapping
-        return self._get_existing_mapping()
-
     def _get_source_mapping(self):
         cldr_mapping = proppluralunit._get_language_mapping(self._store.sourcelanguage)
         if cldr_mapping:
@@ -651,7 +657,7 @@ class proppluralunit(base.TranslationUnit):
     def _get_ordered_units(self):
         # Used for str (GWT order)
         mapping = self.personality.get_expand_output_target_mapping(
-            self._get_target_mapping()
+            self._store.get_plural_tags()
         )
         names = [
             name for name in self.personality.get_cldr_names_order() if name in mapping
@@ -670,11 +676,11 @@ class proppluralunit(base.TranslationUnit):
         elif isinstance(text, list):
             strings = text
         elif isinstance(text, dict):
-            mapping, strings = map(list, zip(*text.items()))
+            mapping, strings = map(list, zip(*text.items(), strict=True))
         else:
             strings = [text]
         if mapping is None:
-            mapping = self._get_target_mapping()
+            mapping = self._store.get_plural_tags()
 
         strings = self._get_strings(strings, mapping)
         units = self._get_units(mapping)
@@ -685,11 +691,11 @@ class proppluralunit(base.TranslationUnit):
         if "other" not in mapping and "other" in self.units:
             self.units["other"].target = strings[-1]
 
-        for a, b in zip(strings, units):
+        for a, b in zip(strings, units, strict=True):
             b.target = a
 
     def gettarget(self):
-        ll = [x.target for x in self._get_units(self._get_target_mapping())]
+        ll = [x.target for x in self._get_units(self._store.get_plural_tags())]
         if len(ll) > 1:
             return multistring(ll)
         return ll[0]
@@ -709,7 +715,7 @@ class proppluralunit(base.TranslationUnit):
         elif isinstance(text, list):
             strings = text
         elif isinstance(text, dict):
-            mapping, strings = tuple(map(list, zip(*text.items())))
+            mapping, strings = tuple(map(list, zip(*text.items(), strict=True)))
         else:
             strings = [text]
         if mapping is None:
@@ -722,7 +728,7 @@ class proppluralunit(base.TranslationUnit):
                 f'Not same plural counts between "{strings}" and "{units}"'
             )
 
-        for a, b in zip(strings, units):
+        for a, b in zip(strings, units, strict=True):
             b.source = a
 
     source = property(getsource, setsource)
@@ -866,18 +872,18 @@ class propunit(base.TranslationUnit):
         # symbol that should end every property sentence
         # (e.g. ";" is required for Mac OS X strings)
         self.out_ending = getattr(self.personality, "out_ending", "")
-        self.explicitely_missing = False
+        self.explicitly_missing = False
         self.output_missing = False
 
     @property
     def missing(self):
-        return self.explicitely_missing or (
+        return self.explicitly_missing or (
             not bool(self.translation) and not bool(self.source)
         )
 
     @missing.setter
     def missing(self, missing):
-        self.explicitely_missing = missing
+        self.explicitly_missing = missing
 
     @staticmethod
     def get_missing_part():
@@ -911,7 +917,7 @@ class propunit(base.TranslationUnit):
     def target(self, target):
         self._rich_target = None
         self.translation = self.personality.encode(target or "", self.encoding)
-        self.explicitely_missing = not bool(target)
+        self.explicitly_missing = not bool(target)
 
     @property
     def encoding(self):
@@ -1068,7 +1074,7 @@ class propfile(base.TranslationStore):
 
         for line in propsrc.split("\n"):
             # handle multiline value if we're in one
-            line = quote.rstripeol(line)
+            line = rstripeol(line)
             if inmultilinevalue:
                 newunit.value += line.lstrip()
                 # see if there's more
